@@ -680,7 +680,7 @@ static int apparmor_setprocattr(struct task_struct *task, char *name,
 
 fail:
 	sa.type = LSM_AUDIT_DATA_NONE;
-	aad_set(&sa, &aad);
+	sa.aad = &aad;
 	aad.label = aa_current_label();
 	aad.op = OP_SETPROCATTR;
 	aad.info = name;
@@ -1205,7 +1205,7 @@ module_param_named(paranoid_load, aa_g_paranoid_load, aabool,
 
 /* Boot time disable flag */
 static bool apparmor_enabled = CONFIG_SECURITY_APPARMOR_BOOTPARAM_VALUE;
-module_param_named(enabled, apparmor_enabled, bool, S_IRUGO);
+module_param_named(enabled, apparmor_enabled, bool, S_IRUSR);
 
 /* Boot time to set use of default or unconfined as initial profile */
 bool aa_g_unconfined_init = CONFIG_SECURITY_APPARMOR_UNCONFINED_INIT;
